/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.category.apio.internal.architect.form;

import com.liferay.apio.architect.form.Form;
import com.liferay.apio.architect.form.Form.Builder;

import java.util.Collections;
import java.util.Locale;
import java.util.Map;

/**
 * Instances of this class represent the values extracted from a nested category
 * form.
 *
 * @author Eduardo Perez
 * @author Javier Gamarra
 * @review
 */
public class NestedCategoryForm {

	/**
	 * Builds a {@code Form} that generates {@code NestedCategoryForm} depending
	 * on the HTTP body.
	 *
	 * @param  builder the {@code Form} builder
	 * @return a nested category form
	 * @review
	 */
	public static Form<NestedCategoryForm> buildForm(
		Builder<NestedCategoryForm> builder) {

		return builder.title(
			__ -> "Category form"
		).description(
			__ -> "This form can be used to create or update a category"
		).constructor(
			NestedCategoryForm::new
		).addOptionalLong(
			"category", NestedCategoryForm::_setParentCategoryId
		).addOptionalString(
			"description", NestedCategoryForm::_setDescription
		).addRequiredLong(
			"vocabulary", NestedCategoryForm::_setVocabularyId
		).addRequiredString(
			"name", NestedCategoryForm::_setName
		).build();
	}

	/**
	 * Returns the asset category's description map.
	 *
	 * @return the asset category's description map
	 * @review
	 */
	public Map<Locale, String> getDescriptions(Locale locale) {
		return Collections.singletonMap(locale, _description);
	}

	/**
	 * Returns the asset category's parent category ID.
	 *
	 * @return the asset category's parent category ID
	 * @review
	 */
	public long getParentCategoryId() {
		return _parentCategoryId;
	}

	/**
	 * Returns the asset category's title map.
	 *
	 * @return the asset category's title map
	 * @review
	 */
	public Map<Locale, String> getTitles(Locale locale) {
		return Collections.singletonMap(locale, _name);
	}

	/**
	 * Returns the asset category's vocabulary ID.
	 *
	 * @return the asset category's vocabulary ID
	 * @review
	 */
	public long getVocabularyId() {
		return _vocabularyId;
	}

	private void _setDescription(String description) {
		_description = description;
	}

	private void _setName(String name) {
		_name = name;
	}

	private void _setParentCategoryId(long parentCategoryId) {
		_parentCategoryId = parentCategoryId;
	}

	private void _setVocabularyId(long vocabularyId) {
		_vocabularyId = vocabularyId;
	}

	private String _description;
	private String _name;
	private long _parentCategoryId;
	private long _vocabularyId;

}